package cobol;

import com.proetcon.cobol.rts.CobolProgram;
import com.proetcon.cobol.rts.CobolFile;
import com.proetcon.cobol.rts.exception.CobolRuntimeException;
import com.proetcon.cobol.rts.exception.CobolOverflowException;
import com.proetcon.cobol.rts.types.CobolObject;
import com.proetcon.cobol.rts.types.CobolDataObject;
import com.proetcon.cobol.rts.types.CobolArray;
import com.proetcon.cobol.rts.types.CobolStruct;
import com.proetcon.cobol.rts.types.CobolNumber;
import com.proetcon.cobol.rts.types.CobolString;
import com.proetcon.cobol.rts.util.ParameterStringFactory;
import static com.proetcon.cobol.rts.types.CobolNumber.*;
import static com.proetcon.cobol.rts.types.CobolString.*;
import static com.proetcon.cobol.rts.util.IOUtilities.*;
import static com.proetcon.cobol.rts.util.StringUtilities.*;
import static com.proetcon.cobol.rts.util.MathUtilities.*;
import static com.proetcon.cobol.rts.util.DateUtilities.*;

public class Xm059 extends CobolProgram {

	private static Xm059 INSTANCE = null;
	public CobolString commreg = createString(8, LOW_VALUE);

	public class AcceptDate extends CobolStruct {
		public CobolNumber currentYear = createNumber(2);
		public CobolNumber currentMonth = createNumber(2);
		public CobolNumber currentDay = createNumber(2);

		public void writeString(ParameterStringFactory factoryOut)
				throws CobolRuntimeException {
			factoryOut.store(currentYear);
			factoryOut.store(currentMonth);
			factoryOut.store(currentDay);
		}

		public void readString(ParameterStringFactory factoryIn)
				throws CobolRuntimeException {
			factoryIn.fetch(currentYear);
			factoryIn.fetch(currentMonth);
			factoryIn.fetch(currentDay);
		}
	}

	public AcceptDate acceptDate = new AcceptDate();

	public class AcceptTime extends CobolStruct {
		public CobolNumber currentHour = createNumber(2);
		public CobolNumber currentMinute = createNumber(2);
		public CobolNumber currentSecond = createNumber(2);

		public void writeString(ParameterStringFactory factoryOut)
				throws CobolRuntimeException {
			factoryOut.store(currentHour);
			factoryOut.store(currentMinute);
			factoryOut.store(currentSecond);
		}

		public void readString(ParameterStringFactory factoryIn)
				throws CobolRuntimeException {
			factoryIn.fetch(currentHour);
			factoryIn.fetch(currentMinute);
			factoryIn.fetch(currentSecond);
		}
	}

	public AcceptTime acceptTime = new AcceptTime();

	public class CurrentDate extends CobolStruct {
		public CobolNumber currentYear = createNumber(2);
		public CobolString filler_1 = createString(1, "/");
		public CobolNumber currentMonth = createNumber(2);
		public CobolString filler_2 = createString(1, "/");
		public CobolNumber currentDay = createNumber(2);

		public void writeString(ParameterStringFactory factoryOut)
				throws CobolRuntimeException {
			factoryOut.store(currentYear);
			factoryOut.store(filler_1);
			factoryOut.store(currentMonth);
			factoryOut.store(filler_2);
			factoryOut.store(currentDay);
		}

		public void readString(ParameterStringFactory factoryIn)
				throws CobolRuntimeException {
			factoryIn.fetch(currentYear);
			factoryIn.fetch(filler_1);
			factoryIn.fetch(currentMonth);
			factoryIn.fetch(filler_2);
			factoryIn.fetch(currentDay);
		}
	}

	public CurrentDate currentDate = new CurrentDate();

	public class TimeOfDay extends CobolStruct {
		public CobolNumber currentHour = createNumber(2);
		public CobolString filler_3 = createString(1, "/");
		public CobolNumber currentMinute = createNumber(2);
		public CobolString filler_4 = createString(1, "/");
		public CobolNumber currentSecond = createNumber(2);

		public void writeString(ParameterStringFactory factoryOut)
				throws CobolRuntimeException {
			factoryOut.store(currentHour);
			factoryOut.store(filler_3);
			factoryOut.store(currentMinute);
			factoryOut.store(filler_4);
			factoryOut.store(currentSecond);
		}

		public void readString(ParameterStringFactory factoryIn)
				throws CobolRuntimeException {
			factoryIn.fetch(currentHour);
			factoryIn.fetch(filler_3);
			factoryIn.fetch(currentMinute);
			factoryIn.fetch(filler_4);
			factoryIn.fetch(currentSecond);
		}
	}

	public TimeOfDay timeOfDay = new TimeOfDay();

	//**************************************************************

	public class Ddmmyy extends CobolStruct {
		public CobolNumber dd = createNumber(2);
		public CobolNumber month = createNumber(2);
		public CobolNumber century = createNumber(2);

		//  Redefining Field Length does not match Redefined!
		public CobolString ceX = createString(3);
		{
			ceX.redefines(century);
		}

		public CobolNumber year = createNumber(2);

		//  Redefining Field Length does not match Redefined!
		public CobolString yyX = createString(3);
		{
			yyX.redefines(year);
		}

		public void writeString(ParameterStringFactory factoryOut)
				throws CobolRuntimeException {
			factoryOut.store(dd);
			factoryOut.store(month);
			factoryOut.store(century);
			factoryOut.store(ceX);
			factoryOut.store(year);
			factoryOut.store(yyX);
		}

		public void readString(ParameterStringFactory factoryIn)
				throws CobolRuntimeException {
			factoryIn.fetch(dd);
			factoryIn.fetch(month);
			factoryIn.fetch(century);
			factoryIn.fetch(ceX);
			factoryIn.fetch(year);
			factoryIn.fetch(yyX);
		}
	}

	public Ddmmyy ddmmyy = new Ddmmyy();
	public CobolString p16 = createString(1);

	public class DatumNumbers extends CobolStruct {
		public CobolNumber ddp = createNumber(3);
		public CobolNumber mmp = createNumber(3);
		public CobolNumber yyp = createNumber(3);
		public CobolNumber ddb = createSignedNumber(4, SIGN.TRAILING);
		public CobolNumber mmb = createSignedNumber(4, SIGN.TRAILING);
		public CobolNumber yyb = createSignedNumber(4, SIGN.TRAILING);

		public void writeString(ParameterStringFactory factoryOut)
				throws CobolRuntimeException {
			factoryOut.store(ddp);
			factoryOut.store(mmp);
			factoryOut.store(yyp);
			factoryOut.store(ddb);
			factoryOut.store(mmb);
			factoryOut.store(yyb);
		}

		public void readString(ParameterStringFactory factoryIn)
				throws CobolRuntimeException {
			factoryIn.fetch(ddp);
			factoryIn.fetch(mmp);
			factoryIn.fetch(yyp);
			factoryIn.fetch(ddb);
			factoryIn.fetch(mmb);
			factoryIn.fetch(yyb);
		}
	}

	public DatumNumbers datumNumbers = new DatumNumbers();

	public class WorkData1 extends CobolStruct {
		public CobolNumber sprachencode = createNumber(1);
		public CobolString leftRightAlignment = createString(1);
		public CobolNumber doppelt = createSignedNumber(15, SIGN.TRAILING, 0);
		public CobolNumber ytt = createSignedNumber(8, SIGN.TRAILING, -1);
		public CobolNumber stt = createSignedNumber(8, SIGN.TRAILING, 0);
		public CobolNumber mtt = createSignedNumber(8, SIGN.TRAILING, 0);
		public CobolNumber ttt = createSignedNumber(8, SIGN.TRAILING, 0);

		public void writeString(ParameterStringFactory factoryOut)
				throws CobolRuntimeException {
			factoryOut.store(sprachencode);
			factoryOut.store(leftRightAlignment);
			factoryOut.store(doppelt);
			factoryOut.store(ytt);
			factoryOut.store(stt);
			factoryOut.store(mtt);
			factoryOut.store(ttt);
		}

		public void readString(ParameterStringFactory factoryIn)
				throws CobolRuntimeException {
			factoryIn.fetch(sprachencode);
			factoryIn.fetch(leftRightAlignment);
			factoryIn.fetch(doppelt);
			factoryIn.fetch(ytt);
			factoryIn.fetch(stt);
			factoryIn.fetch(mtt);
			factoryIn.fetch(ttt);
		}
	}

	public WorkData1 workData1 = new WorkData1();

	public class Ww extends CobolStruct {
		public CobolArray<CobolString> w1 = new CobolArray<CobolString>(10,
				createString(1));

		public void writeString(ParameterStringFactory factoryOut)
				throws CobolRuntimeException {
			factoryOut.store(w1);
		}

		public void readString(ParameterStringFactory factoryIn)
				throws CobolRuntimeException {
			factoryIn.fetch(w1);
		}
	}

	public Ww ww = new Ww();

	//**************************************************************

	public class Wotab extends CobolStruct {
		public CobolString filler_5 = createString(30,
				"DIENSTAG  MARDI     MARTEDI   ");
		public CobolString filler_6 = createString(30,
				"MITTWOCH  MERCREDI  MERCOLEDI ");
		public CobolString filler_7 = createString(30,
				"DONNERSTAGJEUDI     GIOVEDI   ");
		public CobolString filler_8 = createString(30,
				"FREITAG   VENDREDI  VENERDI   ");
		public CobolString filler_9 = createString(30,
				"SAMSTAG   SAMEDI    SABATO    ");
		public CobolString filler_10 = createString(30,
				"SONNTAG   DIMANCHE  DOMENICA  ");
		public CobolString filler_11 = createString(30,
				"MONTAG    LUNDI     LUNEDI    ");

		public void writeString(ParameterStringFactory factoryOut)
				throws CobolRuntimeException {
			factoryOut.store(filler_5);
			factoryOut.store(filler_6);
			factoryOut.store(filler_7);
			factoryOut.store(filler_8);
			factoryOut.store(filler_9);
			factoryOut.store(filler_10);
			factoryOut.store(filler_11);
		}

		public void readString(ParameterStringFactory factoryIn)
				throws CobolRuntimeException {
			factoryIn.fetch(filler_5);
			factoryIn.fetch(filler_6);
			factoryIn.fetch(filler_7);
			factoryIn.fetch(filler_8);
			factoryIn.fetch(filler_9);
			factoryIn.fetch(filler_10);
			factoryIn.fetch(filler_11);
		}
	}

	public Wotab wotab = new Wotab();

	public class WotabRedef extends CobolStruct {

		public class Days3 extends CobolStruct {
			public CobolString lang1 = createString(10);
			public CobolString lang2 = createString(10);
			public CobolString lang3 = createString(10);

			public void writeString(ParameterStringFactory factoryOut)
					throws CobolRuntimeException {
				factoryOut.store(lang1);
				factoryOut.store(lang2);
				factoryOut.store(lang3);
			}

			public void readString(ParameterStringFactory factoryIn)
					throws CobolRuntimeException {
				factoryIn.fetch(lang1);
				factoryIn.fetch(lang2);
				factoryIn.fetch(lang3);
			}
		}

		public CobolArray<Days3> days3 = new CobolArray<Days3>(7, new Days3());

		public void writeString(ParameterStringFactory factoryOut)
				throws CobolRuntimeException {
			factoryOut.store(days3);
		}

		public void readString(ParameterStringFactory factoryIn)
				throws CobolRuntimeException {
			factoryIn.fetch(days3);
		}
	}

	public WotabRedef wotabRedef = new WotabRedef();
	{
		wotabRedef.redefines(wotab);
	}

	public class Tatab extends CobolStruct {
		public CobolNumber filler_12 = createSignedNumber(4, SIGN.TRAILING, 0);

		//  DC    H'31'
		public CobolNumber filler_13 = createSignedNumber(4, SIGN.TRAILING, 31);

		//  DC    H'59'
		public CobolNumber filler_14 = createSignedNumber(4, SIGN.TRAILING, 59);

		//  DC    H'90'
		public CobolNumber filler_15 = createSignedNumber(4, SIGN.TRAILING, 90);

		//  DC    H'120'
		public CobolNumber filler_16 = createSignedNumber(4, SIGN.TRAILING, 120);

		//  DC    H'151'
		public CobolNumber filler_17 = createSignedNumber(4, SIGN.TRAILING, 151);

		//  DC    H'181'
		public CobolNumber filler_18 = createSignedNumber(4, SIGN.TRAILING, 181);

		//  DC    H'212'
		public CobolNumber filler_19 = createSignedNumber(4, SIGN.TRAILING, 212);

		//  DC    H'243'
		public CobolNumber filler_20 = createSignedNumber(4, SIGN.TRAILING, 243);

		//  DC    H'273'
		public CobolNumber filler_21 = createSignedNumber(4, SIGN.TRAILING, 273);

		//  DC    H'304'
		public CobolNumber filler_22 = createSignedNumber(4, SIGN.TRAILING, 304);

		//  DC    H'334'
		public CobolNumber filler_23 = createSignedNumber(4, SIGN.TRAILING, 334);

		//  DC    X'FF'
		public CobolNumber filler_24 = createSignedNumber(4, SIGN.TRAILING, 336);

		public void writeString(ParameterStringFactory factoryOut)
				throws CobolRuntimeException {
			factoryOut.store(filler_12);
			factoryOut.store(filler_13);
			factoryOut.store(filler_14);
			factoryOut.store(filler_15);
			factoryOut.store(filler_16);
			factoryOut.store(filler_17);
			factoryOut.store(filler_18);
			factoryOut.store(filler_19);
			factoryOut.store(filler_20);
			factoryOut.store(filler_21);
			factoryOut.store(filler_22);
			factoryOut.store(filler_23);
			factoryOut.store(filler_24);
		}

		public void readString(ParameterStringFactory factoryIn)
				throws CobolRuntimeException {
			factoryIn.fetch(filler_12);
			factoryIn.fetch(filler_13);
			factoryIn.fetch(filler_14);
			factoryIn.fetch(filler_15);
			factoryIn.fetch(filler_16);
			factoryIn.fetch(filler_17);
			factoryIn.fetch(filler_18);
			factoryIn.fetch(filler_19);
			factoryIn.fetch(filler_20);
			factoryIn.fetch(filler_21);
			factoryIn.fetch(filler_22);
			factoryIn.fetch(filler_23);
			factoryIn.fetch(filler_24);
		}
	}

	public Tatab tatab = new Tatab();

	//  DC    X'FF'

	public class TatabRed extends CobolStruct {
		public CobolArray<CobolNumber> tatab1 = new CobolArray<CobolNumber>(12,
				createNumber(4));

		public void writeString(ParameterStringFactory factoryOut)
				throws CobolRuntimeException {
			factoryOut.store(tatab1);
		}

		public void readString(ParameterStringFactory factoryIn)
				throws CobolRuntimeException {
			factoryIn.fetch(tatab1);
		}
	}

	public TatabRed tatabRed = new TatabRed();
	{
		tatabRed.redefines(tatab);
	}

	public class WorkData2 extends CobolStruct {
		public CobolNumber r1 = createSignedNumber(8, SIGN.TRAILING);
		public CobolNumber r9 = createSignedNumber(8, SIGN.TRAILING);
		public CobolNumber r8 = createSignedNumber(8, SIGN.TRAILING);
		public CobolNumber tabI = createSignedNumber(4, SIGN.TRAILING);
		public CobolNumber wI = createSignedNumber(4, SIGN.TRAILING);

		public void writeString(ParameterStringFactory factoryOut)
				throws CobolRuntimeException {
			factoryOut.store(r1);
			factoryOut.store(r9);
			factoryOut.store(r8);
			factoryOut.store(tabI);
			factoryOut.store(wI);
		}

		public void readString(ParameterStringFactory factoryIn)
				throws CobolRuntimeException {
			factoryIn.fetch(r1);
			factoryIn.fetch(r9);
			factoryIn.fetch(r8);
			factoryIn.fetch(tabI);
			factoryIn.fetch(wI);
		}
	}

	public WorkData2 workData2 = new WorkData2();

	public class Param1 extends CobolStruct {
		public CobolNumber param1Tag = createNumber(2);
		public CobolString filler_25 = createString(1);
		public CobolNumber param1Monat = createNumber(2);
		public CobolString filler_26 = createString(1);
		public CobolNumber param1Jahrhundert = createNumber(2);
		public CobolString filler_27 = createString(1);
		public CobolNumber param1Jahr = createNumber(2);

		public void writeString(ParameterStringFactory factoryOut)
				throws CobolRuntimeException {
			factoryOut.store(param1Tag);
			factoryOut.store(filler_25);
			factoryOut.store(param1Monat);
			factoryOut.store(filler_26);
			factoryOut.store(param1Jahrhundert);
			factoryOut.store(filler_27);
			factoryOut.store(param1Jahr);
		}

		public void readString(ParameterStringFactory factoryIn)
				throws CobolRuntimeException {
			factoryIn.fetch(param1Tag);
			factoryIn.fetch(filler_25);
			factoryIn.fetch(param1Monat);
			factoryIn.fetch(filler_26);
			factoryIn.fetch(param1Jahrhundert);
			factoryIn.fetch(filler_27);
			factoryIn.fetch(param1Jahr);
		}
	}

	public Param1 param1 = new Param1();

	public class Param2 extends CobolStruct {
		public CobolNumber langCode = createNumber(1);

		public void writeString(ParameterStringFactory factoryOut)
				throws CobolRuntimeException {
			factoryOut.store(langCode);
		}

		public void readString(ParameterStringFactory factoryIn)
				throws CobolRuntimeException {
			factoryIn.fetch(langCode);
		}
	}

	public Param2 param2 = new Param2();

	public class Param3 extends CobolStruct {
		public CobolString direction = createString(1);

		public void writeString(ParameterStringFactory factoryOut)
				throws CobolRuntimeException {
			factoryOut.store(direction);
		}

		public void readString(ParameterStringFactory factoryIn)
				throws CobolRuntimeException {
			factoryIn.fetch(direction);
		}
	}

	public Param3 param3 = new Param3();

	public class Param4 extends CobolStruct {
		public CobolString dayName = createString(10);

		public void writeString(ParameterStringFactory factoryOut)
				throws CobolRuntimeException {
			factoryOut.store(dayName);
		}

		public void readString(ParameterStringFactory factoryIn)
				throws CobolRuntimeException {
			factoryIn.fetch(dayName);
		}
	}

	public Param4 param4 = new Param4();

	public static void call() throws CobolRuntimeException {
		Xm059.getInstance().callProgram();
	}

	public static void call(Object... parameters) throws CobolRuntimeException {
		Xm059.getInstance().callProgram(parameters);
	}

	private Xm059() {
	}

	public static void main(String[] args) {
		Xm059.getInstance().run(args);
	}

	public static Xm059 getInstance() {
		if (INSTANCE == null) {
			INSTANCE = new Xm059();
		}
		return INSTANCE;
	}

	@Override
	public CobolObject[] getParameters() {
		return (new CobolObject[] { param1, param2, param3, param4 });
	}

	@Override
	public CobolObject getReturning() {
		return null;
	}

	public void procedure() throws CobolRuntimeException {
		param1.param1Jahrhundert.setValue(19);
		ddmmyy.dd.setValue(param1.param1Tag);
		ddmmyy.month.setValue(param1.param1Monat);
		ddmmyy.year.setValue(param1.param1Jahr);
		workData1.sprachencode.setValue(param2.langCode);

		//  CALL  XM016,(DDMMYY,P16),VL
		workData1.leftRightAlignment.setValue(param3.direction);
		p16.setValue("0");
		Xm016.call(byReference(ddmmyy), byReference(p16));
		if (!isEqual(p16, "0")) {
			param4.dayName.setValue("?");
			stop();
		}

		//                            DEFAULT WERT
		if (workData1.sprachencode.getValue() == 1
				|| workData1.sprachencode.getValue() == 2
				|| workData1.sprachencode.getValue() == 3) {
		} else {

			//                            DEFAULT WERT
			workData1.sprachencode.setValue(1);
		}
		if (isEqual(workData1.leftRightAlignment, "L")
				|| isEqual(workData1.leftRightAlignment, "R")) {
		} else {
			workData1.leftRightAlignment.setValue("L");
		}
		datumNumbers.ddp.setValue(ddmmyy.dd);
		datumNumbers.mmp.setValue(ddmmyy.month);
		datumNumbers.yyp.setValue(ddmmyy.year);
		datumNumbers.ddb.setValue(datumNumbers.ddp);

		//**************************************************************
		//                            YEAR
		//                            MINUS 1
		//                            GET DAYS
		//                            SAVE IT
		datumNumbers.mmb.setValue(datumNumbers.mmp);

		//**************************************************************
		//        KORREKTUR SCHALTJAHRE
		datumNumbers.yyb.setValue(datumNumbers.yyp);

		//                            YEAR / 4
		workData1.ytt.compute((datumNumbers.yyb.getValue() - 1) * 365, false);
		workData2.r9.setValue(datumNumbers.yyb);
		workData2.r8.setValue(datumNumbers.yyb);

		//                            /     SCHALTJAHR
		workData2.r9.compute(workData2.r9.getValue() / 4, false);

		//                            / MONTH > FEBR. OK
		//                            SUBTRACT 1
		workData2.r8.compute(workData2.r8.getValue() - 4
				* workData2.r9.getValue(), false);
		if (workData2.r8.getValue() == 0 && datumNumbers.mmb.getValue() <= 2) {

			//                            SAVE CORRECTION
			workData2.r9.compute(workData2.r9.getValue() - 1, false);
		}

		//                            SAVE DAYS FROM PREVIOS MONTHS
		workData1.stt.setValue(workData2.r9);
		workData2.tabI.setValue(datumNumbers.mmb);
		if (workData2.tabI.getValue() < 1 || workData2.tabI.getValue() > 12) {
			display("ERROR IN XM059, TATAB INDEX: " + workData2.tabI.toString()
					+ "  ERDOES");
			param4.dayName.setValue("X");
			stop();
		}
		workData1.mtt.setValue(tatabRed.tatab1.get(workData2.tabI));
		workData2.r9.compute(workData1.ytt.getValue()
				+ workData1.stt.getValue() + workData1.mtt.getValue()
				+ datumNumbers.ddb.getValue(), false);
		workData2.r1.compute(workData2.r9.getValue() / 7, false);
		workData2.r8.compute(7 * workData2.r1.getValue(), false);
		workData2.tabI.compute(workData2.r9.getValue()
				- workData2.r8.getValue(), false);
		if (workData2.tabI.getValue() == 0) {
			workData2.tabI.setValue(7);
		}
		if (workData2.tabI.getValue() < 1 || workData2.tabI.getValue() > 7) {
			display("ERROR IN XM059, WOTAB INDEX: " + workData2.tabI.toString()
					+ "  ERDOES");
			param4.dayName.setValue("X");

			//                            MOVE WEEKDAY TO USER
			stop();
		}
		if (workData1.sprachencode.getValue() == 1) {
			param4.dayName.setValue(wotabRedef.days3.get(workData2.tabI).lang1);
		} else if (workData1.sprachencode.getValue() == 2) {
			param4.dayName.setValue(wotabRedef.days3.get(workData2.tabI).lang2);
		} else if (workData1.sprachencode.getValue() == 3) {

			//                            FOR LEFT SCHIFT
			param4.dayName.setValue(wotabRedef.days3.get(workData2.tabI).lang3);
		}
		if (isEqual(workData1.leftRightAlignment, "L")) {
		} else {
			ww.setValue(param4.dayName);
			workData2.tabI.setValue(10);
			while (!!isEqual(ww.w1.get(workData2.tabI), " ")) {
				workData2.tabI.compute(workData2.tabI.getValue() - 1, false);
			}
			for (workData2.wI.setValue(10); !(workData2.tabI.getValue() == 0 || workData2.tabI
					.getValue() == 10); workData2.wI.increment(-1)) {
				ww.w1.get(workData2.wI).setValue(ww.w1.get(workData2.tabI));
				ww.w1.get(workData2.tabI).setValue(" ");
				workData2.tabI.compute(workData2.tabI.getValue() - 1, false);
			}
			param4.dayName.setValue(ww);
		}
		stop();
	}

}

/**** JGen: unassigned comments ****/

//                            GET ADDR FOR DAYS
//**************************************************************

