      $SET ANS85
       IDENTIFICATION DIVISION.
       PROGRAM-ID. LIEFAUFT.
      ******************************************************************
      *  PROGRAM NAME: LIEFAUFT.                                       *
      *  SYSTEM  NAME: AUFTRAG = AUFTRAGBEARBEITUNG                    *
      *  PURPOSE:      subprogram for producing the supplier orders    *
      *  VERSION:      1.1 (07.96)                                     *
      *  LANGUAGE:     MICROSOFT (R) COBOL OPTIMIZIG COMPILER          *
      *                VERSION 3.00A ("MICROFOCUS COBOL")              *
      ******************************************************************
      *  PROGRAM INPUTS:                                               *
      *    - Datei der Lieferposten von Auftragsverarbeitung           *
      *    - Indexed sequentielle Datei der Lieferanten                *
      *    - Indexed sequentielle Datei der Lieferantenpreise          *
      *  PROGRAM OUTPUTS:                                              *
      *    - Liste der Auftrge an Lieferanten                         *
      *  PROGRAM WORK FILES:                                           *
      *    - Datei zur Sortierung der Lieferposten                     *
      *  PROGRAM FUNCTIONS:                                            *
      *    - open  Lieferpostendatei                                   *
      *    - open  Lieferposten-Workfile                               *
      *    - Do for each Lieferposten                                  *
      *      - read  Lieferposten                                      *
      *      - select Lieferant                                        *
      *      - write Lieferposten Workfile                             *
      *    - EndLoop                                                   *
      *    - close Lieferpostendatei                                   *
      *    - close Lieferposten-Workfile.                              *
      ******************************************************************
      *    - sort  Lieferposten Workfile                               *
      ******************************************************************
      *    - open  Lieferposten Workfile                               *
      *    - init  Lieferauftragsdatei                                 *
      *    - Do for each Lieferant                                     *
      *      - init  Lieferauftrag                                     *
      *      - Do for each Lieferposten                                *
      *        - write Lieferposten Workfile                           *
      *      - EndLoop                                                 *
      *      - sum   Lieferauftrag                                     *
      *    - Endloop                                                   *
      *    - close Lieferposten Workfile                               *
      *    - close Lieferauftragsdatei                                 *
      ******************************************************************
       ENVIRONMENT DIVISION.
       CONFIGURATION SECTION.
       SOURCE-COMPUTER. IBM-PC.
       OBJECT-COMPUTER. IBM-PC.
       INPUT-OUTPUT SECTION.
       FILE-CONTROL.
      *  Datei der Lieferposten von Auftragsverarbeitung
          SELECT RESUPPLY-FILE ASSIGN TO INPUT-FILE
                 ORGANIZATION IS LINE SEQUENTIAL
                 FILE STATUS IS RESUPPLY-STATUS.
      *  Datei der sortierten Lieferposten
          SELECT RESUPPLY-WORKFILE ASSIGN TO 'Outputs\WorkFile.txt'
                 ORGANIZATION IS LINE SEQUENTIAL
                 FILE STATUS IS WORKFILE-STATUS.
      *  Datei zur Sortierung der Lieferposten
          SELECT RESUPPLY-SORTFILE ASSIGN TO 'Outputs\SortFile.txt'
                 SORT STATUS IS SORTFILE-STATUS.
      *  Liste der Auftrge an Lieferanten
          SELECT LIEFERAUFTRAGSDATEI ASSIGN TO 'Outputs\Liefauft.txt'
                 ORGANIZATION IS LINE SEQUENTIAL
                 FILE STATUS IS LIEFAUFT-STATUS.
      ******************************************************************
      *
       DATA DIVISION.
      *
      ******************************************************************
      *
       FILE SECTION.
      *
      ******************************************************************
      *
      *  Datei der Lieferposten von Auftragsverarbeitung
      *
       FD  RESUPPLY-FILE
           DATA RECORD IS RESUPPLY-ITEM.
      ****************************************************************
      * Resupply Orders are sent to the module processResupplyOrders
WRAP   01  RESUPPLY-ITEM-REC    PIC X(80).
WRAP  * ****  Record removed ****
       FD  RESUPPLY-WORKFILE
           DATA RECORD IS WORKREC.
       01  WORKREC.
           05  WORK-REC-TYPE                 PIC XX.
           05  WORK-SUPPLIER-NO              PIC 9(6).
           05  WORK-ART-NO                   PIC 9(6).
           05  WORK-ART-NAME                 PIC X(30).
           05  WORK-ART-AMOUNT               PIC 9999.
           05  WORK-ART-PRICE                PIC 9999.99.
      *
      *  Datei zur Sortierung der Lieferposten
      *
       SD  RESUPPLY-SORTFILE
           RECORD CONTAINS 54 CHARACTERS
           DATA RECORD IS SORTREC.
       01  SORTREC.
           05  SORT-REC-TYPE                 PIC XX.
           05  SORT-SUPPLIER-NO              PIC 9(6).
           05  SORT-ART-NO                   PIC 9(6).
           05  SORT-ART-NAME                 PIC X(30).
           05  SORT-ART-AMOUNT               PIC 9999.
           05  SORT-ART-PRICE                PIC 9999.99.
      *
      *  Liste der Auftrge an Lieferanten
      *
       FD  LIEFERAUFTRAGSDATEI
           DATA RECORD IS AUFTRAGSSATZ.
       01  AUFTRAGSSATZ.
           05  AUFTRAGSZEILE                 PIC X(72).
      *
      ******************************************************************
      *
       WORKING-STORAGE SECTION.
      *
      ******************************************************************
      *
      *  File Status Fields
kludge 77  WORKLIB-PATH                      PIC X(40)
           vALUE 'c:\DATA\SAMPLES\ORDERS'.
       77  DB-DIRECTORY                      PIC X(80)
           vALUE 'Database'.
       77  INPUT-FILE                        PIC X(80).
       77  RESUPPLY-STATUS                   PIC XX.
           88  OK-RESUPPLY                   VALUE ZERO.
           88  EOF-RESUPPLY                  VALUE '09'.
       77  WORKFILE-NAME                     PIC X(80).
       77  WORKFILE-STATUS                   PIC XX.
           88  OK-WORKFILE                   VALUE ZERO.
           88  EOF-WORKFILE                  VALUE '09'.
       77  SORTFILE-NAME                     PIC X(80).
       77  SORTFILE-STATUS                   PIC XX.
           88  OK-SORTFILE                   VALUE ZERO.
           88  EOF-SORTFILE                  VALUE '09'.
       77  LIEFAUFT-STATUS                   PIC XX.
           88  OK-LIEFAUFT                   VALUE ZERO.
           88  EOF-LIEFAUFT                  VALUE '09'.
       77  FEHLER-CODE                       PIC 99.
           88  AUFTRAG-OK                    VALUE ZERO.
           88  BAD-LIEFERPOSTEN              VALUE 1.
           88  NO-OFFER                      VALUE 2.
           88  NO-LIEFERANT                  VALUE 3.
       77  CONTINUE-KEY                      PIC XX.
      ************************************************************
      *
       01  DB-INTERFACE.
           05  DB-TABLE-NAME              PIC X(8).
           05  DB-OPERATION               PIC X(8).
           05  DB-SEARCH-KEY              PIC 9(6).
           05  DB-RECORD                  PIC X(92).
           05  DB-RETURN-CODE             PIC 99.
      *
      ************************************************************
      *
       01  SUPPLIER-RECORD.
           05  SUPPLIER-REC-TYPE             PIC XX.
           05  FILLER                        PIC X.
           05  SUPPLIER-NO                   PIC 9(6).
           05  FILLER                        PIC X.
           05  SUPPLLIER-TIME                PIC 99.
           05  FILLER                        PIC X.
           05  SUPPLIER-MODE                 PIC X(4).
               88  LUFT                      VALUE 'AIR '.
               88  BAHN                      VALUE 'RAIL'.
               88  SCHIFF                    VALUE 'SHIP'.
               88  LKW                       VALUE 'ROAD'.
           05  FILLER                        PIC X.
           05  SUPPLIER-DATA.
               10  SUPPLIER-NAME             PIC X(30).
               10  FILLER                    PIC X.
               10  SUPPLIER-ADDRESS.
                   15  SUPPLIER-ZIP          PIC 9(4).
                   15  FILLER                PIC X.
                   15  SUPPLIER-CITY         PIC X(20).
                   15  FILLER                PIC X.
                   15  SUPPLIER-STREET       PIC X(30).
                   15  FILLER                PIC X.
                   15  SUPPLIER-STATE        PIC X(4).
      *
       01  PRICE-RECORD.
           05  PRICE-REC-TYPE                PIC XX.
           05  FILLER                        PIC X.
           05  PRICE-ART-NO                  PIC 9(6).
           05  FILLER                        PIC X.
           05  PRICE-SUPPLIER-NO             PIC 9(6).
           05  FILLER                        PIC X.
           05  PRICE-ART-PRICE               PIC 9999.99.
           05  FILLER                        PIC X.
           05  PRICE-SUPPLIER-NAME           PIC X(30).
      *
      ************************************************************
      *
       01  LISTENKONSTANTEN.
           05  ZEILEN-MAX                    PIC 99  VALUE 72.
           05  SEITEN-MAX                    PIC 99  VALUE 99.
       01  LISTENVARIABELN.
           05  TAGESDATUM.
               10  JAHR                      PIC 99.
               10  MONAT                     PIC 99.
               10  TAG                       PIC 99.
           05  LISTENDATUM.
               10  LISTENTAG                 PIC 99.
               10  FILLER                    PIC X   VALUE '/'.
               10  LISTENMONAT               PIC 99.
               10  FILLER                    PIC X   VALUE '/'.
               10  LISTENJAHR                PIC 99.
           05  SEITENNR                      PIC 999 VALUE 1.
           05  ZEILENANZAHL                  PIC 99  VALUE 73.
           05  LETZTE-LIEFERANTNR            PIC 9(6) VALUE ZEROES.
SNEED      05  ART-PRICE                     PIC 9999V99.
SNEED      05  ART-PRICE-DEC   REDEFINES ART-PRICE.
SNEED          10  ART-PRICE-WHOLE           PIC 9999.
SNEED          10  ART-PRICE-FRACT           PIC 99.
       01  LIEFERBEDINGUNGEN.
           05  LIEFERMODUS-1               PIC X(30) VALUE
              'Lieferung mit Luftfracht  '.
           05  LIEFERMODUS-2               PIC X(30) VALUE
              'Lieferung mit der Bahn    '.
           05  LIEFERMODUS-3               PIC X(30) VALUE
              'Lieferung mit Schiff      '.
           05  LIEFERMODUS-4               PIC X(30) VALUE
              'Lieferung mit LKW         '.
           05  LIEFERANTENPROGRAMM           PIC X(10) VALUE 'Liefern'.
       01  LIEFERAUFTRAGSKOPF.
           05  KOPFZEILE-1.
               10  FILLER                    PIC X     VALUE '+'.
               10  FILLER                    PIC X(70) VALUE ALL '-'.
               10  FILLER                    PIC X     VALUE '+'.
           05  KOPFZEILE-2.
               10  FILLER                    PIC X     VALUE '|'.
               10  FILLER                    PIC X(70) VALUE SPACES.
               10  FILLER                    PIC X     VALUE '|'.
           05  KOPFZEILE-3.
               10  FILLER                    PIC X     VALUE '|'.
               10  FILLER                    PIC X(2)  VALUE SPACES.
               10  FILLER                    PIC X(8) VALUE 'TITEL:  '.
               10  KOPFZEILE-TITEL           PIC X(42).
               10  FILLER                    PIC X(8) VALUE 'SEITE:  '.
               10  KOPFZEILE-SEITE           PIC 9(4).
               10  FILLER                    PIC X(6) VALUE SPACES.
               10  FILLER                    PIC X     VALUE '|'.
           05  KOPFZEILE-4.
               10  FILLER                    PIC X     VALUE '|'.
               10  FILLER                    PIC X(2)  VALUE SPACES.
               10  FILLER                    PIC X(8) VALUE 'QUELLE: '.
               10  KOPFZEILE-PROGRAM         PIC X(42).
               10  KOPFZEILE-4-4             PIC X(8) VALUE 'DATUM:  '.
               10  KOPFZEILE-DATUM           PIC X(8).
               10  FILLER                    PIC X(2) VALUE SPACES.
               10  FILLER                    PIC X     VALUE '|'.
           05  TITELZEILE-7                  PIC X(72).
           05  TITELZEILE-8.
               10  FILLER                    PIC X     VALUE '+'.
               10  FILLER                    PIC X(70) VALUE ALL '-'.
               10  FILLER                    PIC X     VALUE '+'.
           05  FUSSZEILE-1.
               10  FILLER                    PIC X     VALUE '+'.
               10  FILLER                    PIC X(70) VALUE ALL '-'.
               10  FILLER                    PIC X     VALUE '+'.
           05  FUSSZEILE-2                   PIC X(72).
           05  FUSSZEILE-3                   PIC X(72).
           05  FUSSZEILE-4                   PIC X(72).
       01  LIEFERVARIABELN.
           05  BEST-PREIS                    PIC 9999.99 VALUE 9999.99.
           05  BEST-LIEFERANT                PIC 9(6) VALUE ZERO.
           05  AUFTRAGSNR                    PIC 9(6) VALUE ZERO.
           05  LIEF-POSTEN-NR                PIC 99 VALUE ZERO.
           05  RECHENWERTE.
               10  GESAMTPREIS               PIC 99999V99 VALUE ZERO.
               10  LIEFERMENGE               PIC 999 VALUE ZERO.
               10  LIEFERBETRAG              PIC 9999999V99 VALUE ZERO.
       01  LIEFERAUFTRAGSTITEL.
           05  FILLER                        PIC X     VALUE '|'.
           05  FILLER                        PIC X(70) VALUE
           '  Rechnung der Universitt Regensburg              '.
           05  FILLER                        PIC X     VALUE '|'.
       01  LIEFERAUFTRAGSKOPF-1.
           05  FILLER                        PIC X     VALUE '|'.
           05  FILLER                        PIC X(6)  VALUE SPACES.
           05  FILLER                        PIC X(18) VALUE
                                            '        Kundenkz: '.
           05  LIEFERAUFTRAGSKOPF-LIEF-KZ    PIC 9(6).
           05  FILLER                        PIC X(6)  VALUE SPACES.
           05  FILLER                        PIC X(18) VALUE
                                            'Lieferauftragsnr: '.
           05  LIEFERAUFTRAGSKOPF-AUFTRAGNR  PIC X(8).
           05  FILLER                        PIC X(8)  VALUE SPACES.
           05  FILLER                        PIC X     VALUE '|'.
       01  LIEFERAUFTRAGSKOPF-2.
           05  FILLER                        PIC X     VALUE '|'.
           05  FILLER                        PIC X(6)  VALUE SPACES.
           05  FILLER                        PIC X(18) VALUE
                                            'Lieferantenname : '.
           05  LIEFERAUFTRAGSKOPF-NAME       PIC X(30).
           05  FILLER                        PIC X(16) VALUE SPACES.
           05  FILLER                        PIC X     VALUE '|'.
       01  LIEFERAUFTRAGSKOPF-3.
           05  FILLER                        PIC X     VALUE '|'.
           05  FILLER                        PIC X(2)  VALUE SPACES.
           05  FILLER                        PIC X(22) VALUE
                                            'Lieferantenanschrift: '.
           05  LIEFERAUFTRAGSKOPF-STRASSE    PIC X(30).
           05  FILLER                        PIC X(16) VALUE SPACES.
           05  FILLER                        PIC X     VALUE '|'.
       01  LIEFERAUFTRAGSKOPF-4.
           05  FILLER                        PIC X     VALUE '|'.
           05  FILLER                        PIC X(24) VALUE SPACES.
           05  LIEFERAUFTRAGSKOPF-LAND       PIC X(4).
           05  LIEFERAUFTRAGSKOPF-PLZ        PIC X(6).
           05  LIEFERAUFTRAGSKOPF-ORT        PIC X(30).
           05  FILLER                        PIC X(6)  VALUE SPACES.
           05  FILLER                        PIC X     VALUE '|'.
       01  LIEFERAUFTRAGSKOPF-5.
           05  FILLER                        PIC X     VALUE '|'.
           05  FILLER                        PIC X(6)  VALUE SPACES.
           05  FILLER                        PIC X(18) VALUE
                                            ' Lieferungsmodus: '.
           05  LIEFERAUFTRAGSKOPF-LIEFMODUS  PIC X(30).
           05  FILLER                        PIC X(16) VALUE SPACES.
           05  FILLER                        PIC X     VALUE '|'.
       01  LIEFERPOSTENTITEL.
           05  FILLER                        PIC X    VALUE '|'.
           05  FILLER                        PIC X(9) VALUE 'Bestellnr'.
           05  FILLER                        PIC X    VALUE SPACES.
           05  FILLER                        PIC X(9) VALUE 'Artikelnr'.
           05  FILLER                        PIC X    VALUE SPACES.
           05  FILLER                        PIC X(20) VALUE
              ' Artikelbezeichnung '.
           05  FILLER                        PIC X(7) VALUE SPACES.
           05  FILLER                        PIC X(5) VALUE 'Menge'.
           05  FILLER                        PIC XX   VALUE SPACES.
           05  FILLER                        PIC X(5) VALUE 'Preis'.
           05  FILLER                        PIC XXX  VALUE SPACES.
           05  FILLER                        PIC X(5) VALUE 'Summe'.
           05  FILLER                        PIC XXX  VALUE SPACES.
           05  FILLER                        PIC X     VALUE '|'.
       01  LIEFERPOSTENZEILE.
           05  FILLER                        PIC X     VALUE '|'.
           05  FILLER                        PIC X(2)  VALUE SPACES.
           05  LIEFERZEILE-BESTELLNR         PIC X(6).
           05  FILLER                        PIC X(2) VALUE SPACES.
           05  LIEFERZEILE-ARTIKELNR         PIC X(6).
           05  FILLER                        PIC X(2) VALUE SPACES.
           05  LIEFERZEILE-ARTIKELNAME       PIC X(20).
           05  FILLER                        PIC X(6) VALUE SPACES.
           05  LIEFERZEILE-BESTELLMENGE      PIC ZZZZZ9.
           05  FILLER                        PIC X(2) VALUE SPACES.
           05  LIEFERZEILE-STUECKPREIS       PIC ZZZ9.99.
           05  FILLER                        PIC X    VALUE SPACES.
           05  LIEFERZEILE-GESAMTPREIS       PIC ZZ,Z99.99.
           05  FILLER                        PIC X    VALUE SPACE.
           05  FILLER                        PIC X     VALUE '|'.
       01  LIEFERAUFTRAGSFUSS-1.
           05  FILLER                        PIC X     VALUE '|'.
           05  FILLER                        PIC X(10) VALUE SPACES.
           05  FILLER                        PIC X(18) VALUE
                                            'Bestellungsanzahl:'.
           05  LIEFERAUFTRAGSFUSS-MENGE      PIC ZZZ9.
           05  FILLER                        PIC X(9)  VALUE SPACES.
           05  FILLER                        PIC X(18) VALUE
                                            'Lieferungsbetrag: '.
           05  LIEFERAUFTRAGSFUSS-BETRAG     PIC ZZZ,ZZ9.99.
           05  FILLER                        PIC X     VALUE SPACE.
           05  FILLER                        PIC X     VALUE '|'.
       01  LIEFERAUFTRAGSFUSS-2.
           05  FILLER                        PIC X     VALUE '|'.
           05  FILLER                        PIC X(10) VALUE SPACES.
           05  FILLER                        PIC X(52) VALUE
               'Die bestellte Ware ist innerhalb 30 Tage zu liefern '.
           05  FILLER                        PIC X(8)  VALUE SPACE.
           05  FILLER                        PIC X     VALUE '|'.
       01  FEHLERZEILE.
           05  FILLER                        PIC X     VALUE '|'.
           05  FEHLERMELDUNG                 PIC X(70).
           05  FILLER                        PIC X     VALUE '|'.
       01  FEHLERMELDUNGEN.
           05  FM-1                          PIC X(50) VALUE
              '   Lieferpostendaten sind fehlerhaft            '.
           05  FM-2                          PIC X(50) VALUE
              '   Artikel wird nicht mehr geliefert            '.
           05  FM-3                          PIC X(50) VALUE
              '   Lieferant ist nicht mehr bekannt             '.
      ****************************************************************
      *  Parameters from calling interface
      *
       01  CMD-LINE                          PIC X(80).
       01  RESULT                            PIC X  COMP-X.
       01  FUNCT                             PIC X  COMP-X VALUE 35.
       01  CMD-LINED.
           05  CMD-LINE-LNG                  PIC X  COMP-X VALUE ZERO.
      /
      ************************************************************
      ************************************************************
WRAP       COPY RESUPPLY.
WRAP  *************** COBWRAP parameters ******************************
WRAP  *  Parameters for COBOL Programs
WRAP   01 X-COBOL-PARAMS.
WRAP      05  X-REC-LNG                PIC 9(8) USAGE BINARY.
WRAP      05  X-MAP-LNG                PIC 9(8) USAGE BINARY.
WRAP      05  X-ACCEPT-LNG             PIC 9999.
WRAP      05  X-DISPLAY-LNG            PIC 9999.
WRAP      05  X-ACCEPT-TEXT            PIC X(80).
WRAP      05  X-DISPLAY-TEXT           PIC X(80).
WRAP  *****************************************************************
WRAP  *  Parameters for data exchange between Program & Wrapper
WRAP  *
WRAP   01 X-PARAMS.
WRAP      05  X-FUNCODE                PIC XX.
WRAP      05  X-RETCODE                PIC 99.
WRAP      05  X-FILNAME                PIC X(80).
WRAP      05  X-METHOD                 PIC X(30).
********************  End COBWRAP Parameters *************************
       PROCEDURE DIVISION.
      *****************************************************
       LIEFERPOSTENVERARBEITUNGS SECTION.
      *    - open  Lieferpostendatei                                   *
      *    - open  Workfile                                            *
      *    - Do for each Lieferposten                                  *
      *      - read  Lieferposten                                      *
      *      - select Lieferant                                        *
      *      - write Lieferposten Workfile                             *
      *    - EndLoop                                                   *
      *    - close Lieferpostendatei                                   *
      *    - close Workfile                                            *
      *****************************************************
       INIT-LIEFERUNG.
           ACCEPT CMD-LINE FROM COMMAND-LINE.
           MOVE CMD-LINE TO INPUT-FILE
           DISPLAY 'Input Params = ' INPUT-FILE.
TEST***    ACCEPT CONTINUE-KEY.
       OPEN-LIEFERPOSTEN.
           MOVE ZEROES TO RESUPPLY-STATUS.
WRAP  *    OPEN INPUT RESUPPLY-FILE
           IF RESUPPLY-STATUS NOT = ZEROES
              DISPLAY 'RESUPPLY File not found'
              DISPLAY 'Resupply File Name = ' INPUT-FILE
              DISPLAY 'Press Enter to terminate !'
              ACCEPT  CONTINUE-KEY
              GOBACK
           END-IF.
           DISPLAY 'Resupply File was opened '.
           MOVE INPUT-FILE TO WORKFILE-NAME
           INSPECT WORKFILE-NAME REPLACING ALL 'XML' BY 'WRK'
           INSPECT WORKFILE-NAME REPLACING ALL 'xml' BY 'wrk'
           MOVE ZEROES TO WORKFILE-STATUS.
           OPEN OUTPUT RESUPPLY-WORKFILE.
TEST***    DISPLAY 'Workfile opened = ' WORKFILE-NAME
TEST***    ACCEPT CONTINUE-KEY.
      *****************************************************
       READ-LIEFERPOSTEN.
           MOVE ZEROES TO RESUPPLY-STATUS.
           PERFORM UNTIL EOF-RESUPPLY
WRAP  *       READ RESUPPLY-FILE
WRAP  *       AT END
WRAP       MOVE 'RD' TO X-FUNCODE
WRAP       MOVE ZEROES TO X-RETCODE
WRAP       MOVE INPUT-FILE TO X-FILNAME
WRAP       MOVE 'RESUPPLY-FILE' TO X-METHOD
WRAP       MOVE 54 TO X-REC-LNG
WRAP       CALL 'RESUPPLY' USING X-FUNCODE, X-RETCODE,
WRAP                             X-FILNAME, X-METHOD,
WRAP                             X-REC-LNG, RESUPPLY-ITEM
TEST       DISPLAY 'Resupply item is:' RESUPPLY-ITEM (1:60)
TEST       DISPLAY 'Press Enter key to process!'
TEST       ACCEPT CONTINUE-KEY
WRAP       IF X-RETCODE NOT = ZEROES
                 MOVE '09' TO RESUPPLY-STATUS
              ELSE
                 MOVE RESUPPLY-REC-TYPE TO WORK-REC-TYPE
                 MOVE RESUPPLY-ART-NO TO WORK-ART-NO
                 MOVE RESUPPLY-ART-NAME TO WORK-ART-NAME
                 MOVE RESUPPLY-ART-AMOUNT TO WORK-ART-AMOUNT
                 MOVE ZEROES TO WORK-SUPPLIER-NO
                 MOVE ZEROES TO WORK-ART-PRICE
                 PERFORM FIND-LIEFERANT
                 MOVE BEST-LIEFERANT TO WORK-SUPPLIER-NO
                 MOVE BEST-PREIS TO WORK-ART-PRICE
                 WRITE WORKREC
TEST***          DISPLAY 'WorkArtprice = ' WORK-ART-PRICE
TEST***          ACCEPT CONTINUE-KEY
WRAP       END-IF
           END-PERFORM.
           DISPLAY 'Lieferposten copied and Lieferant selected'
           EXIT.
      *****************************************************
       CLOSE-LIEFAUFT.
WRAP       CLOSE RESUPPLY-FILE.
           CLOSE RESUPPLY-WORKFILE.
TEST***    DISPLAY 'WorkFile is at :' WORKFILE-NAME
TEST***    ACCEPT CONTINUE-KEY.
           EXIT.
      *****************************************************************
      *****************************************************************
       SORTIERUNGS      SECTION.
      *    - sort  Lieferposten Workfile                               *
           MOVE INPUT-FILE TO SORTFILE-NAME
           INSPECT SORTFILE-NAME REPLACING ALL 'XML' BY 'SRT'
           INSPECT SORTFILE-NAME REPLACING ALL 'xml' BY 'srt'
TEST***    DISPLAY 'SortFile = ' SORTFILE-NAME
TEST***    ACCEPT CONTINUE-KEY
           MOVE ZERO TO SORTFILE-STATUS
           SORT RESUPPLY-SORTFILE ON ASCENDING KEY SORT-SUPPLIER-NO
                 USING  RESUPPLY-WORKFILE
                 GIVING RESUPPLY-WORKFILE.
           IF NOT OK-SORTFILE
              DISPLAY 'Error on sorting Lieferposten '
              DISPLAY 'Sort File Status:' SORTFILE-STATUS
              DISPLAY 'Press Enter to Continue !!!'
              ACCEPT CONTINUE-KEY
           END-IF.
TEST***    DISPLAY 'Sort Lieferposten by Lieferantennr completed'
TEST***    ACCEPT CONTINUE-KEY.
      *****************************************************************
      *****************************************************************
       LIEFERPOSTENAUSGABE   SECTION.
      *    - open  Lieferposten Workfile                               *
      *    - init  Lieferauftragsdatei                                 *
      *    - Do for each Lieferant                                     *
      *      - init  Lieferauftrag                                     *
      *      - Do for each Lieferposten                                *
      *        - write Lieferposten Workfile                           *
      *      - EndLoop                                                 *
      *      - sum   Lieferauftrag                                     *
      *    - Endloop                                                   *
      *    - close Lieferposten Workfile                               *
      *    - close Lieferauftragsdatei                                 *
      *****************************************************
       INIT-LIEFERAUFTRAG.
           MOVE 'Lieferauftrag   ' TO KOPFZEILE-TITEL.
           MOVE 'Lieferauftragsbearbeitung ' TO KOPFZEILE-PROGRAM.
           PERFORM OPEN-LISTE.
           EXIT.
      *****************************************************
       OPEN-LIEFERPOSTEN-WORKFILE.
           MOVE ZEROES TO WORKFILE-STATUS.
           OPEN INPUT RESUPPLY-WORKFILE.
           IF WORKFILE-STATUS NOT = ZEROES
              DISPLAY 'RESUPPLY-WORKFILE not found'
TEST***       DISPLAY 'Press Enter to terminate !'
TEST***       ACCEPT  CONTINUE-KEY
              GOBACK
           END-IF.
           DISPLAY 'Workfile was reopened '.
      *****************************************************
       READ-SORTIERTE-LIEFERPOSTEN.
           MOVE ZEROES TO WORKFILE-STATUS.
           PERFORM UNTIL EOF-WORKFILE
              READ RESUPPLY-WORKFILE
              AT END
                 MOVE '09' TO WORKFILE-STATUS
              NOT AT END
                 IF WORK-SUPPLIER-NO NOT = LETZTE-LIEFERANTNR
                     IF LETZTE-LIEFERANTNR NOT = ZEROES
                        PERFORM DRUCKE-LIEFERAUFTRAGSFUSS
                     END-IF
                     MOVE ZERO TO FEHLER-CODE
                     PERFORM GET-LIEFERANTENDATEN
                     IF FEHLER-CODE = ZERO
                        PERFORM DRUCKE-LIEFERAUFTRAGSKOPF
                     END-IF
                  END-IF
                  PERFORM DRUCKE-LIEFERPOSTEN
              END-READ
              MOVE WORK-SUPPLIER-NO TO LETZTE-LIEFERANTNR
           END-PERFORM.
           PERFORM DRUCKE-LIEFERAUFTRAGSFUSS.
TEST***    DISPLAY 'Lieferauftraege have been printed'
TEST***    ACCEPT CONTINUE-KEY.
           EXIT.
      *****************************************************
       CLOSE-RECHNUNGSPOSTEN.
           CLOSE RESUPPLY-WORKFILE.
           DELETE FILE RESUPPLY-WORKFILE.
           PERFORM CLOSE-LISTE.
           GOBACK.
      *****************************************************************
      *****************************************************************
       LIEFERAUFTRAGAUSGABE   SECTION.
      *    - write Lieferauftragkopf                                   *
      *    - write Lieferauftragsposten                                *
      *    - write Lieferauftragsfuss                                  *
      *****************************************************
       DRUCKE-LIEFERAUFTRAGSKOPF.
           ADD  1 TO AUFTRAGSNR
           MOVE AUFTRAGSNR TO LIEFERAUFTRAGSKOPF-AUFTRAGNR
           MOVE WORK-SUPPLIER-NO TO LIEFERAUFTRAGSKOPF-LIEF-KZ
           MOVE SUPPLIER-NAME TO LIEFERAUFTRAGSKOPF-NAME
           MOVE SUPPLIER-ZIP  TO LIEFERAUFTRAGSKOPF-PLZ
           MOVE SUPPLIER-CITY TO LIEFERAUFTRAGSKOPF-ORT
           MOVE SUPPLIER-STREET TO LIEFERAUFTRAGSKOPF-STRASSE
           MOVE SUPPLIER-STATE TO LIEFERAUFTRAGSKOPF-LAND
           EVALUATE SUPPLIER-MODE
               WHEN 'AIR '
                   MOVE LIEFERMODUS-3 TO LIEFERAUFTRAGSKOPF-LIEFMODUS
               WHEN 'RAIL'
                   MOVE LIEFERMODUS-2 TO LIEFERAUFTRAGSKOPF-LIEFMODUS
               WHEN 'SHIP'
                   MOVE LIEFERMODUS-1 TO LIEFERAUFTRAGSKOPF-LIEFMODUS
               WHEN OTHER
                   MOVE LIEFERMODUS-4 TO LIEFERAUFTRAGSKOPF-LIEFMODUS
           END-EVALUATE
           PERFORM PRINT-HEADER
           MOVE ZERO TO LIEFERMENGE
           MOVE ZERO TO LIEFERBETRAG
           MOVE ZERO TO LIEF-POSTEN-NR.
      *****************************************************
       DRUCKE-LIEFERPOSTEN.
           ADD 1 TO LIEF-POSTEN-NR
           MOVE LIEF-POSTEN-NR   TO LIEFERZEILE-BESTELLNR
           MOVE WORK-ART-NO      TO LIEFERZEILE-ARTIKELNR
           MOVE WORK-ART-NAME    TO LIEFERZEILE-ARTIKELNAME
           MOVE WORK-ART-AMOUNT  TO LIEFERZEILE-BESTELLMENGE
           ADD  WORK-ART-AMOUNT   TO LIEFERMENGE
           MOVE WORK-ART-PRICE (1:4) TO ART-PRICE-WHOLE
           MOVE WORK-ART-PRICE (6:2) TO ART-PRICE-FRACT
           MOVE ART-PRICE   TO LIEFERZEILE-STUECKPREIS
           MULTIPLY WORK-ART-AMOUNT BY ART-PRICE
              GIVING GESAMTPREIS
           MOVE GESAMTPREIS      TO LIEFERZEILE-GESAMTPREIS
           ADD GESAMTPREIS TO LIEFERBETRAG
           PERFORM PRINT-LINE.
      *****************************************************
       DRUCKE-LIEFERAUFTRAGSFUSS.
           EVALUATE FEHLER-CODE
              WHEN 1
                 MOVE FM-1 TO FEHLERMELDUNG
                 MOVE FEHLERZEILE TO FUSSZEILE-2
              WHEN 2
                 MOVE FM-2 TO FEHLERMELDUNG
                 MOVE FEHLERZEILE TO FUSSZEILE-2
              WHEN 3
                 MOVE FM-3 TO FEHLERMELDUNG
                 MOVE FEHLERZEILE TO FUSSZEILE-2
              WHEN OTHER
                 MOVE LIEFERMENGE TO LIEFERAUFTRAGSFUSS-MENGE
                 MOVE LIEFERBETRAG TO LIEFERAUFTRAGSFUSS-BETRAG
                 MOVE LIEFERAUFTRAGSFUSS-1 TO FUSSZEILE-2
                 MOVE LIEFERAUFTRAGSFUSS-2 TO FUSSZEILE-3
           END-EVALUATE.
           PERFORM PRINT-FOOTING
           PERFORM PAGE-CHANGE.
           EXIT.
      *****************************************************************
      *****************************************************************
       ACCESS-SUBROUTINES SECTION.
      *****************************************************
       FIND-LIEFERANT.
           MOVE 9999.99 TO BEST-PREIS.
           MOVE ZERO TO BEST-LIEFERANT.
           MOVE 'PRICES '  TO DB-TABLE-NAME
           MOVE 'FIRST  '  TO DB-OPERATION
           MOVE SPACES     TO DB-RECORD
           MOVE WORK-ART-NO TO DB-SEARCH-KEY
           CALL 'PRICES' USING DB-INTERFACE, DB-DIRECTORY
           MOVE DB-RECORD TO PRICE-RECORD
           IF DB-RETURN-CODE = ZERO
              MOVE ZERO TO DB-RETURN-CODE
              PERFORM UNTIL DB-RETURN-CODE NOT = ZERO
                 MOVE 'NEXT   '  TO DB-OPERATION
                 CALL 'PRICES' USING DB-INTERFACE, DB-DIRECTORY
                 MOVE DB-RECORD TO PRICE-RECORD
                 IF PRICE-ART-PRICE < BEST-PREIS
                    MOVE PRICE-ART-PRICE TO BEST-PREIS
                    MOVE PRICE-SUPPLIER-NO TO BEST-LIEFERANT
                 END-IF
              END-PERFORM
           ELSE
              MOVE 2 TO FEHLER-CODE
           END-IF.
TEST***    DISPLAY 'Supplier with lowest price selected '
TEST***    DISPLAY 'lowest price = ' BEST-PREIS
TEST***    ACCEPT CONTINUE-KEY.
           EXIT.
      *****************************************************
       GET-LIEFERANTENDATEN.
           MOVE 'SUPPLIER' TO DB-TABLE-NAME
           MOVE 'SELECT '  TO DB-OPERATION
           MOVE SPACES     TO DB-RECORD
           MOVE WORK-SUPPLIER-NO TO DB-SEARCH-KEY
           CALL 'SUPPLIER' USING DB-INTERFACE, DB-DIRECTORY
           MOVE DB-RECORD TO SUPPLIER-RECORD
           IF DB-RETURN-CODE NOT = ZERO
              MOVE 3 TO FEHLER-CODE
           ELSE
              CONTINUE
TEST***       DISPLAY 'Supplier was found '
TEST***       DISPLAY 'Supplier Record is:'  SUPPLIER-RECORD
TEST***       ACCEPT CONTINUE-KEY
           END-IF.
           EXIT.
      *****************************************************************
      *****************************************************************
       LIST-SUBROUTINES SECTION.
      *****************************************************
       OPEN-LISTE.
           OPEN OUTPUT LIEFERAUFTRAGSDATEI.
           ACCEPT TAGESDATUM FROM DATE.
           MOVE TAG TO LISTENTAG.
           MOVE MONAT TO LISTENMONAT.
           MOVE JAHR TO LISTENJAHR.
           MOVE LISTENDATUM TO KOPFZEILE-DATUM.
           MOVE SEITENNR TO KOPFZEILE-SEITE.
           MOVE LIEFERANTENPROGRAMM TO KOPFZEILE-PROGRAM
           MOVE LIEFERAUFTRAGSTITEL TO TITELZEILE-7
           EXIT.
      *****************************************************
       PRINT-HEADER.
           MOVE SEITENNR TO KOPFZEILE-SEITE
           WRITE AUFTRAGSSATZ FROM KOPFZEILE-1
           WRITE AUFTRAGSSATZ FROM KOPFZEILE-2
           WRITE AUFTRAGSSATZ FROM KOPFZEILE-3
           WRITE AUFTRAGSSATZ FROM KOPFZEILE-4
           WRITE AUFTRAGSSATZ FROM KOPFZEILE-2
           WRITE AUFTRAGSSATZ FROM KOPFZEILE-1
           WRITE AUFTRAGSSATZ FROM TITELZEILE-7
           WRITE AUFTRAGSSATZ FROM TITELZEILE-8
           WRITE AUFTRAGSSATZ FROM LIEFERAUFTRAGSKOPF-1
           WRITE AUFTRAGSSATZ FROM LIEFERAUFTRAGSKOPF-2
           WRITE AUFTRAGSSATZ FROM LIEFERAUFTRAGSKOPF-3
           WRITE AUFTRAGSSATZ FROM LIEFERAUFTRAGSKOPF-4
           WRITE AUFTRAGSSATZ FROM LIEFERAUFTRAGSKOPF-5
           WRITE AUFTRAGSSATZ FROM KOPFZEILE-1
           WRITE AUFTRAGSSATZ FROM LIEFERPOSTENTITEL
           WRITE AUFTRAGSSATZ FROM KOPFZEILE-1
           MOVE 16 TO ZEILENANZAHL
           ADD 1 TO SEITENNR.
           EXIT.
      *****************************************************
       PRINT-LINE.
           MOVE LIEFERPOSTENZEILE TO AUFTRAGSZEILE.
           WRITE AUFTRAGSSATZ.
           ADD 1 TO ZEILENANZAHL
           EXIT.
      *****************************************************
       PRINT-FOOTING.
           WRITE AUFTRAGSSATZ FROM FUSSZEILE-1
           WRITE AUFTRAGSSATZ FROM FUSSZEILE-2
           WRITE AUFTRAGSSATZ FROM FUSSZEILE-3
           WRITE AUFTRAGSSATZ FROM FUSSZEILE-1
           ADD 3 TO ZEILENANZAHL
           EXIT.
      *****************************************************
       PAGE-CHANGE.
           MOVE SPACES TO AUFTRAGSSATZ
           MOVE  '/' TO AUFTRAGSSATZ
           WRITE AUFTRAGSSATZ AFTER PAGE.
           EXIT.
      *****************************************************
       CLOSE-LISTE.
           CLOSE LIEFERAUFTRAGSDATEI.
           DISPLAY 'Lieferauftragsdatei wurde erstellt in LiefAuft.txt'
           DISPLAY 'Press Enter to terminate !!!'
           ACCEPT CONTINUE-KEY.
      * end of source
      ******************************************************************
