      ******************************************************************
      *                                                                *
      *   COBOL PROGRAM FOR PROCESSING BACK ORDERS                     *
      *                                                                *
      ******************************************************************
       IDENTIFICATION DIVISION.
       PROGRAM-ID. RUCKSTEL.
      *AUTHOR.     H. SNEED.
      *DATE.       FEB.  2012.
      /
       ENVIRONMENT DIVISION.
       CONFIGURATION SECTION.
       SPECIAL-NAMES.
IBMPC**     CONSOLE IS OUTPUT-DEV.
       INPUT-OUTPUT SECTION.
       FILE-CONTROL.
XML   *   Datei der Rueckstellposten
           SELECT BACKORDR ASSIGN TO 'RuckStel.dat'
           ORGANIZATION IS LINE SEQUENTIAL
           FILE STATUS IS RUECKSTELLPOSTEN-STAT.
      *
           SELECT LOG-FILE
           ASSIGN TO DYNAMIC LOG-FILE-NAME
           ORGANIZATION IS LINE SEQUENTIAL
           ACCESS IS SEQUENTIAL
           FILE STATUS IS LOG-STAT.
XML   *
           SELECT RUCKSTELLPROTOKOLL
           ASSIGN TO DYNAMIC OUTPUT-FILE
           ORGANIZATION IS LINE SEQUENTIAL
           ACCESS IS SEQUENTIAL
           FILE STATUS IS OUTPUT-STAT.
      *
      ******************************************************************
      /
       DATA DIVISION.
       FILE SECTION.
      *   Datei der Rueckstellposten
       FD  BACKORDR
           DATA RECORD IS BACK-ORDER.
      ****************************************************************
      * Back Orders are sent to the module processBackOrders
WRAP   01  BACK-ORDER-REC    PIC X(80).
WRAP  * ****  Record removed ****
       FD  LOG-FILE
           RECORD CONTAINS 92 CHARACTERS
           DATA RECORD IS LOG-RECORD.
       01  LOG-RECORD                     PIC X(80).
      *
       FD  RUCKSTELLPROTOKOLL
           RECORD CONTAINS 80 CHARACTERS
           DATA RECORD IS PROTOKOLL-REC.
       01  PROTOKOLL-REC                  PIC X(80).
      *
      /
       WORKING-STORAGE SECTION.
      ****************************************************************
      *
       77  RUECKSTELLPOSTEN-STAT          PIC XX.
           88  OK-RUECKSTELLPOSTEN           VALUE ZERO.
           88  EOF-RUECKSTELLPOSTEN          VALUE '09'.
       77  LOG-FILE-NAME                  PIC X(80).
       77  LOG-STAT                       PIC XX.
           88 LOG-OK                      VALUE '00'.
           88 LOG-EOF                     VALUE '09'.
       77  OUTPUT-FILE                    PIC X(80).
       77  OUTPUT-STAT                    PIC XX.
           88 OUTPUT-OK                   VALUE '00'.
           88 OUTPUT-EOF                  VALUE '09'.
       77  DB-DIRECTORY                   PIC X(80)
           VALUE 'DATABASE'.
       77  OUTPUT-DIRECTORY               PIC X(80)
           VALUE 'PENDING'.
       77  OUTPUT-LIB                     PIC X(80)
           VALUE 'OUTPUTS'.
       77  INPUT-FILE                     PIC X(80).
       77  ERROR-CODE                     PIC 99 VALUE ZERO.
       77  START-SW                       PIC 9 VALUE ZERO.
       77  CONTINUE-KEY                   PIC X.
      *
       01  YYMMDD.
           05  YY                       PIC 99.
           05  MM                       PIC 99.
           05  DD                       PIC 99.
      *
       01  CURRENT-DATE.
           10  FILLER                   PIC XX VALUE '20'.
           10  YEAR-OUT                 PIC 99.
           10  MONTH-OUT                PIC 99.
           10  DAY-OUT                  PIC 99.
      *
       01  INDEXES.
           02  I                        PIC 999 VALUE ZERO.
           02  J                        PIC 999 VALUE ZERO.
      *
       01  CONSTANTS.
           02  MAX-ORDER-ITEMS          PIC 99 VALUE 10.
           02  MINIMUM-CREDIT           PIC 99 VALUE 02.
           02  ORDER-ITEM-COUNT         PIC 99 VALUE ZERO.
           02  ORDER-ITEM-SENT          PIC 99 VALUE ZERO.
      *
      ****************************************************************
      *
       01  DC-INTERFACE.
           05  SEND-FUNC                  PIC X(4).
           05  SEND-RETCODE               PIC 99.
      *
      ****************************************************************
      *
       01  DB-INTERFACE.
           05  DB-TABLE-NAME              PIC X(8).
           05  DB-OPERATION               PIC X(8).
           05  DB-SEARCH-KEY              PIC 9(6).
           05  DB-RECORD                  PIC X(92).
           05  DB-RETURN-CODE             PIC 99.
      ****************************************************************
      *
       01  CUSTOMER-RECORD.
           02  CUST-REC-TYPE              PIC XX VALUE 'CU'.
           02  FILLER                     PIC X.
           02  CUST-NO                    PIC 9(6).
           02  FILLER                     PIC X.
           02  CUST-NAME                  PIC X(20).
           02  FILLER                     PIC X.
             02  CUST-CREDIT              PIC 99.
           02  FILLER                     PIC X.
           02  CUST-ADDRESS.
               03  CUST-STREET            PIC X(20).
               03  FILLER                 PIC X.
               03  CUST-ZIP               PIC 9(4).
               03  FILLER                 PIC X.
               03  CUST-CITY              PIC X(20).
               03  FILLER                 PIC X.
               03  CUST-STATE             PIC X(4).
      ****************************************************************
      *
       01  ARTICLE-RECORD.
           02  ART-REC-TYPE               PIC XX VALUE 'AR'.
           02  FILLER                     PIC X.
           02  ART-NO                     PIC 9(6).
           02  FILLER                     PIC X.
           02  ART-TYPE                   PIC X(4).
           02  FILLER                     PIC X.
           02  ART-AMOUNT                 PIC 9(6).
           02  FILLER                     PIC X.
           02  ART-MINIMUM-AMOUNT         PIC 9(6).
           02  FILLER                     PIC X.
           02  ART-RESUPPLY-AMOUNT        PIC 9(6).
           02  FILLER                     PIC X.
           02  ART-STATUS                 PIC XX.
           02  FILLER                     PIC X.
           02  ART-PRICE                  PIC 9999V99.
           02  FILLER                     PIC X.
           02  ART-NAME                   PIC X(30).
      ****************************************************************
      *  Dispatch Orders are sent to the module processDispatchOrders
       01  DISPATCH-ORDER.
           02  DISP-REC-TYPE              PIC XX VALUE 'DI'.
           02  DISP-DATE                  PIC X(8).
           02  DISP-ORDER-NO              PIC 9(6).
           02  DISP-CUST-NO               PIC 9(6).
           02  DISP-CUST-NAME             PIC X(20).
           02  DISP-DELIVERY-MODE         PIC X(4).
           02  DISP-CUST-ADDRESS.
               03  DISP-CUST-STREET       PIC X(30).
               03  DISP-CUST-ZIP          PIC 9(4).
               03  DISP-CUST-CITY         PIC X(20).
               03  DISP-CUST-STATE        PIC X(4).
           02  DISPATCH-ITEM.
               03  DISP-ITEM-NO           PIC 99.
               03  DISP-ART-NO            PIC 9(6).
               03  DISP-ART-NAME          PIC X(30).
               03  DISP-ART-AMOUNT        PIC 9(6).
           02  FILLER                     PIC X(4).
      ****************************************************************
      * Resupply Orders are sent to the module processResupplyOrders
       01  RESUPPLY-ITEM.
           02  RESUPPLY-REC-TYPE          PIC XX VALUE 'RE'.
           02  RESUPPLY-DATE              PIC 9(8).
           02  RESUPPLY-ART-NO            PIC 9(6).
           02  RESUPPLY-ART-TYPE          PIC X(4).
           02  RESUPPLY-ART-AMOUNT        PIC 9(6).
           02  FILLER                     PIC X(4).
      ****************************************************************
      * Customer Billing Items are sent to the module processBilling
       01  BILLING-ITEM.
           02  BILL-REC-TYPE              PIC XX VALUE 'BI'.
           02  BILL-DATE                  PIC 9(8).
           02  BILL-ORDER-NO              PIC 9(6).
           02  BILL-CUST-NO               PIC 9(6).
           02  BILL-ITEM-NO               PIC 9(6).
           02  BILL-ART-NO                PIC 9(6).
           02  BILL-ART-NAME              PIC X(30).
           02  BILL-ART-PRICE             PIC 9999.99.
           02  BILL-ART-AMOUNT            PIC 9(6).
           02  BILL-PAYMENT-MODE          PIC X(8).
           02  FILLER                     PIC X(4).
      ****************************************************************
      *
       01  LOG-HEADER.
           02  FILLER                     PIC XX    VALUE '| '.
           02  FILLER                     PIC X(8)  VALUE 'OrderNo '.
           02  FILLER                     PIC X(8)  VALUE 'CustNo  '.
           02  FILLER                     PIC X(20) VALUE 'CustName'.
           02  FILLER                     PIC X(8)  VALUE 'ArtNo   '.
           02  FILLER                     PIC X(8)  VALUE 'Status  '.
           02  FILLER                     PIC X(8)  VALUE 'Date =  '.
           02  LOG-DATE                   PIC X(8)  VALUE SPACES.
           02  FILLER                     PIC X(8)  VALUE SPACES.
           02  FILLER                     PIC XX    VALUE ' |'.
      *
       01  LOG-DATA.
           02  FILLER                     PIC XX    VALUE '| '.
           02  LOG-ORDER-NO               PIC X(6).
           02  FILLER                     PIC XX    VALUE SPACES.
           02  LOG-CUST-NO                PIC X(6).
           02  FILLER                     PIC XX    VALUE SPACES.
           02  LOG-CUST-NAME              PIC X(20).
           02  LOG-ITEM-NO                PIC X(6).
           02  FILLER                     PIC XX    VALUE SPACES.
           02  LOG-ITEM-STATUS            PIC X(32).
           02  FILLER                     PIC XX    VALUE ' |'.
      ****************************************************************
      *
       01  PROTOKOLLKOPF-0.
           02  FILLER                     PIC X     VALUE ' '.
           02  FILLER                     PIC XX    VALUE '| '.
           02  FILLER                     PIC X(75) VALUE SPACES.
           02  FILLER                     PIC XX    VALUE ' |'.
       01  PROTOKOLLKOPF-1.
           02  FILLER                     PIC X     VALUE ' '.
           02  FILLER                     PIC XX    VALUE '| '.
           02  FILLER                     PIC X(10) VALUE SPACES.
           02  FILLER                     PIC X(48) VALUE
              'Protokoll der erfuellten Rueckstellposten von '.
           02  DATUM-OUT                  PIC X(8)  VALUE SPACES.
           02  FILLER                     PIC X(9)  VALUE SPACES.
           02  FILLER                     PIC XX    VALUE ' |'.
      *
       01  PROTOKOLLKOPF-2.
           02  FILLER                     PIC X     VALUE ' '.
           02  FILLER                     PIC XX    VALUE '| '.
           02  FILLER                     PIC X(6)  VALUE 'AftgNr'.
           02  FILLER                     PIC X     VALUE '|'.
           02  FILLER                     PIC X(6)  VALUE 'KundNr'.
           02  FILLER                     PIC X     VALUE '|'.
           02  FILLER                     PIC X(20) VALUE 'KundenName'.
           02  FILLER                     PIC X     VALUE '|'.
           02  FILLER                     PIC X(6)  VALUE 'ArtNr '.
           02  FILLER                     PIC X     VALUE '|'.
           02  FILLER                     PIC X(6)  VALUE 'Menge '.
           02  FILLER                     PIC X     VALUE '|'.
           02  FILLER                     PIC X(26) VALUE 'Artikelname'.
           02  FILLER                     PIC XX    VALUE ' |'.
      *
       01  PROTOKOLLKOPF-3.
           02  FILLER                     PIC X     VALUE ' '.
           02  FILLER                     PIC XX    VALUE '+-'.
           02  FILLER                     PIC X(6)  VALUE '------'.
           02  FILLER                     PIC X     VALUE '+'.
           02  FILLER                     PIC X(6)  VALUE '------'.
           02  FILLER                     PIC X     VALUE '+'.
           02  FILLER                     PIC X(20) VALUE ALL '-'.
           02  FILLER                     PIC X     VALUE '+'.
           02  FILLER                     PIC X(6)  VALUE '------'.
           02  FILLER                     PIC X     VALUE '+'.
           02  FILLER                     PIC X(6)  VALUE '------'.
           02  FILLER                     PIC X     VALUE '+'.
           02  FILLER                     PIC X(26) VALUE ALL '-'.
           02  FILLER                     PIC XX    VALUE '-+'.
      *
       01  PROTOKOLLDATEN.
           02  FILLER                     PIC X     VALUE ' '.
           02  FILLER                     PIC XX    VALUE '| '.
           02  AUFTRAGNR-OUT              PIC 9(6).
           02  FILLER                     PIC X     VALUE '|'.
           02  KUNDENNR-OUT               PIC 9(6).
           02  FILLER                     PIC X     VALUE '|'.
           02  KUNDENNAME-OUT             PIC X(20).
           02  FILLER                     PIC X     VALUE '|'.
           02  ARTNR-OUT                  PIC 9(6).
           02  FILLER                     PIC X     VALUE '|'.
           02  MENGE-OUT                  PIC 9(6).
           02  FILLER                     PIC X     VALUE '|'.
           02  ARTNAME-OUT                PIC X(26).
           02  FILLER                     PIC XX    VALUE ' |'.
      *
      ****************************************************************
      *
       01  ERROR-MESSAGES.
           02  ERROR-MESSAGE-1            PIC X(32)  VALUE
               'Order cannot be processed'.
           02  ERROR-MESSAGE-2            PIC X(32)  VALUE
               'Customer is not registered'.
           02  ERROR-MESSAGE-3            PIC X(32)  VALUE
               'Customer is not credible'.
           02  ERROR-MESSAGE-4            PIC X(32)  VALUE
               'Order has no items'.
           02  ERROR-MESSAGE-5            PIC X(32)  VALUE
               'Article is not on stock'.
           02  ERROR-MESSAGE-6            PIC X(32)  VALUE
               'Article amount is insufficient'.
           02  ERROR-MESSAGE-7            PIC X(32)  VALUE
               'Back Order could not be sent!'.
           02  ERROR-MESSAGE-8            PIC X(32)  VALUE
               'Dispatch Order could not be sent'.
           02  ERROR-MESSAGE-9            PIC X(32)  VALUE
               'Resupply Order could not be sent'.
           02  ERROR-MESSAGE-10           PIC X(32)  VALUE
               'Billing Item could not be sent'.
           02  ERROR-MESSAGE-11           PIC X(32)  VALUE
               'Ordered article will be sent'.
      ****************************************************************
      *  Parameters from calling interface
      *
       01  CMD-LINE                          PIC X(80).
       01  RESULT                            PIC X  COMP-X.
       01  FUNCT                             PIC X  COMP-X VALUE 35.
       01  CMD-LINED.
           05  CMD-LINE-LNG                  PIC X  COMP-X VALUE ZERO.
      /
      ****************************************************************
WRAP       COPY BACKORDR.
WRAP  *************** COBWRAP parameters ******************************
WRAP  *  Parameters for COBOL Programs
WRAP   01 X-COBOL-PARAMS.
WRAP      05  X-REC-LNG                PIC 9(8) USAGE BINARY.
WRAP      05  X-MAP-LNG                PIC 9(8) USAGE BINARY.
WRAP      05  X-ACCEPT-LNG             PIC 9999.
WRAP      05  X-DISPLAY-LNG            PIC 9999.
WRAP      05  X-ACCEPT-TEXT            PIC X(80).
WRAP      05  X-DISPLAY-TEXT           PIC X(80).
WRAP  *****************************************************************
WRAP  *  Parameters for data exchange between Program & Wrapper
WRAP  *
WRAP   01 X-PARAMS.
WRAP      05  X-FUNCODE                PIC XX.
WRAP      05  X-RETCODE                PIC 99.
WRAP      05  X-FILNAME                PIC X(80).
WRAP      05  X-METHOD                 PIC X(30).
********************  End COBWRAP Parameters *************************
       PROCEDURE DIVISION.
      ****************************************************************
       INITIALIZE-ORDER-PROCESSING SECTION.
       RECEIVE-PARAMS.
      * receive input message from user interface
           ACCEPT CMD-LINE FROM COMMAND-LINE.
           MOVE CMD-LINE TO INPUT-FILE.
           DISPLAY 'Start back Order Processing'.
           DISPLAY 'Input File = ' INPUT-FILE.
TEST***    ACCEPT CONTINUE-KEY.
      ****************************************************************
      * set current date
       SET-CURRENT-DATE.
           ACCEPT YYMMDD FROM DATE
           MOVE YY TO YEAR-OUT
           MOVE MM TO MONTH-OUT
           MOVE DD TO DAY-OUT.
           MOVE CURRENT-DATE TO LOG-DATE.
           MOVE 'ORDERS.log' TO LOG-FILE-NAME.
      ****************************************************************
      * open the log file if first message
       OPEN-LOG-RESPONSES.
           IF START-SW = ZERO
              MOVE SPACES TO LOG-FILE-NAME
              STRING OUTPUT-LIB                DELIMITED BY SPACE
                    '\'                        DELIMITED BY SIZE
                    'Orders.log'               DELIMITED BY SIZE
                 INTO LOG-FILE-NAME
              END-STRING
              OPEN OUTPUT LOG-FILE
              MOVE LOG-HEADER TO LOG-RECORD
              WRITE LOG-RECORD
              MOVE ALL '-' TO LOG-RECORD
              MOVE '+' TO LOG-RECORD (1:1)
              MOVE '+' TO LOG-RECORD (80:1)
              WRITE LOG-RECORD
           ELSE
              OPEN EXTEND LOG-FILE
           END-IF.
      ****************************************************************
      * open the protocol of back orders
       OPEN-PROTOCOL.
           MOVE SPACES TO OUTPUT-FILE
           STRING OUTPUT-LIB                DELIMITED BY SPACE
                 '\'                        DELIMITED BY SIZE
                 'Orders.txt'               DELIMITED BY SIZE
              INTO OUTPUT-FILE
           END-STRING
           OPEN OUTPUT RUCKSTELLPROTOKOLL
           MOVE ALL '-' TO PROTOKOLL-REC
           MOVE ' '  TO PROTOKOLL-REC (1:1)
           MOVE '+'  TO PROTOKOLL-REC (2:1)
           MOVE '+'  TO PROTOKOLL-REC (80:1)
           MOVE CURRENT-DATE TO DATUM-OUT
           WRITE PROTOKOLL-REC
           MOVE PROTOKOLLKOPF-0 TO PROTOKOLL-REC
           WRITE PROTOKOLL-REC
           MOVE PROTOKOLLKOPF-1 TO PROTOKOLL-REC
           WRITE PROTOKOLL-REC
           MOVE PROTOKOLLKOPF-0 TO PROTOKOLL-REC
           WRITE PROTOKOLL-REC
           MOVE PROTOKOLLKOPF-2 TO PROTOKOLL-REC
           WRITE PROTOKOLL-REC
           MOVE PROTOKOLLKOPF-3 TO PROTOKOLL-REC
           WRITE PROTOKOLL-REC.
      *****************************************************
       OPEN-RUECKSTELLPOSTEN.
           MOVE ZEROES TO RUECKSTELLPOSTEN-STAT.
WRAP  *    OPEN INPUT BACKORDR
           IF RUECKSTELLPOSTEN-STAT NOT = ZEROES
              DISPLAY 'BackOrder File could not found'
              DISPLAY 'BackOrder File = ' INPUT-FILE
              DISPLAY 'Press Enter to terminate !'
              ACCEPT  CONTINUE-KEY
              GOBACK
           END-IF.
      *****************************************************
       READ-RUECKSTELLPOSTEN.
           MOVE ZEROES TO RUECKSTELLPOSTEN-STAT.
           PERFORM UNTIL EOF-RUECKSTELLPOSTEN
WRAP  *       READ BACKORDR
WRAP  *       AT END MOVE '09' TO RUECKSTELLPOSTEN-STAT
WRAP       MOVE 'RD' TO X-FUNCODE
WRAP       MOVE ZEROES TO X-RETCODE
WRAP       MOVE INPUT-FILE TO X-FILNAME
WRAP       MOVE 'BACKORDR' TO X-METHOD
WRAP       MOVE 160 TO X-REC-LNG
WRAP       CALL 'BACKORDR ' USING X-FUNCODE, X-RETCODE,
WRAP                             X-FILNAME, X-METHOD,
WRAP                             X-REC-LNG, BACK-ORDER
WRAP       IF X-RETCODE NOT = ZEROES
                     MOVE '09' TO RUECKSTELLPOSTEN-STAT
              ELSE
                  IF BACK-ORDER-REC-TYPE = 'BO'
                     PERFORM PROCESS-BACK-ORDER
                  ELSE
                     DISPLAY 'Invalid Record Type = ' DISP-REC-TYPE
                     DISPLAY 'Press Enter to acknowledge!'
                     ACCEPT  CONTINUE-KEY
                  END-IF
WRAP       END-IF
           END-PERFORM.
           EXIT.
      *****************************************************
       CLOSE-RUECKSTELLPOSTEN.
           MOVE PROTOKOLLKOPF-3 TO PROTOKOLL-REC
           WRITE PROTOKOLL-REC
           CLOSE RUCKSTELLPROTOKOLL.
           DISPLAY '| Back-Order processing terminated!'
           DISPLAY '| Order items processed = ' ORDER-ITEM-COUNT
TEST       ACCEPT CONTINUE-KEY.
WRAP  *    CLOSE BACKORDR.
           GOBACK.
      *****************************************************************
       PROCESS-BACK-ORDER  SECTION.
      ****************************************************************
      * log the back order
       LOG-BACK-ORDER.
      ****************************************************************
      * log the back order
           MOVE BACK-ORDER-NO TO LOG-ORDER-NO
           MOVE BACK-CUST-NO TO LOG-CUST-NO
           MOVE BACK-CUST-NAME TO LOG-CUST-NAME
           MOVE CURRENT-DATE TO LOG-DATE
           MOVE ZEROES TO LOG-ITEM-NO
           MOVE SPACES TO LOG-ITEM-STATUS
TEST       DISPLAY 'BackOrder received:' BACK-ORDER (1:60)
TEST       DISPLAY 'Press Enter to process:'
TEST       ACCEPT CONTINUE-KEY.
      *
      ****************************************************************
      * check if customer is registered
       CHECK-CUSTOMER.
TEST***    ACCEPT CONTINUE-KEY.
           DISPLAY 'Check Customer:' DB-RETURN-CODE
           MOVE 'CUSTOMER' TO DB-TABLE-NAME
           MOVE 'SELECT '  TO DB-OPERATION
           MOVE SPACES     TO DB-RECORD
           MOVE BACK-CUST-NO TO DB-SEARCH-KEY
           CALL 'CUSTOMER' USING DB-INTERFACE, DB-DIRECTORY
           MOVE DB-RECORD TO CUSTOMER-RECORD
           IF DB-RETURN-CODE NOT = ZERO
              MOVE BACK-ORDER-NO TO LOG-ORDER-NO
              MOVE BACK-CUST-NO TO LOG-CUST-NO
              MOVE BACK-CUST-NAME TO LOG-CUST-NAME
              MOVE ZEROES TO LOG-ITEM-NO
              MOVE ERROR-MESSAGE-2 TO LOG-ITEM-STATUS
              MOVE LOG-DATA TO LOG-RECORD
              WRITE LOG-RECORD
TEST          DISPLAY 'Customer is no longer known'
TEST          DISPLAY 'Press enter to continue'
TEST          ACCEPT CONTINUE-KEY
              GOBACK
           END-IF.
      ****************************************************************
      * check if customer is credible
TEST***    DISPLAY 'Check Customer Credit' CUST-CREDIT
TEST***    ACCEPT CONTINUE-KEY
           IF CUST-CREDIT < MINIMUM-CREDIT
              MOVE BACK-ORDER-NO TO LOG-ORDER-NO
              MOVE BACK-CUST-NO TO LOG-CUST-NO
              MOVE BACK-CUST-NAME TO LOG-CUST-NAME
              MOVE SPACES TO LOG-ITEM-NO
              MOVE ERROR-MESSAGE-3 TO LOG-ITEM-STATUS
              MOVE LOG-DATA TO LOG-RECORD
              WRITE LOG-RECORD
TEST***       DISPLAY 'Customer credit rating has dropped'
TEST***       ACCEPT CONTINUE-KEY
           END-IF.
      ****************************************************************
      * check if customer is credible
       PROCESS-ORDER-ITEMS.
TEST***    DISPLAY 'Process Order Items'
TEST***    ACCEPT CONTINUE-KEY
           MOVE 1 TO ORDER-ITEM-COUNT
           MOVE 0 TO ORDER-ITEM-SENT
           IF BACK-ITEM-NO NOT NUMERIC
              MOVE BACK-ORDER-NO TO LOG-ORDER-NO
              MOVE BACK-CUST-NO TO LOG-CUST-NO
              MOVE BACK-CUST-NAME TO LOG-CUST-NAME
              MOVE ZEROES TO LOG-ITEM-NO
              MOVE ERROR-MESSAGE-4 TO LOG-ITEM-STATUS
              MOVE LOG-DATA TO LOG-RECORD
              WRITE LOG-RECORD
TEST***       DISPLAY 'Invalid Order Item '
TEST***       ACCEPT CONTINUE-KEY
           END-IF.
      ****************************************************************
      * process back order
           MOVE 'ARTICLE' TO DB-TABLE-NAME
           MOVE 'SELECT '  TO DB-OPERATION
           MOVE SPACES     TO DB-RECORD
           MOVE BACK-ART-NO TO DB-SEARCH-KEY
           CALL 'ARTICLE' USING DB-INTERFACE, DB-DIRECTORY
           MOVE DB-RECORD TO ARTICLE-RECORD
           IF DB-RETURN-CODE NOT = ZERO
              MOVE BACK-ORDER-NO TO LOG-ORDER-NO
              MOVE BACK-CUST-NO TO LOG-CUST-NO
              MOVE BACK-CUST-NAME TO LOG-CUST-NAME
              MOVE BACK-ART-NO TO LOG-ITEM-NO
              MOVE ERROR-MESSAGE-5 TO LOG-ITEM-STATUS
              MOVE LOG-DATA TO LOG-RECORD
              WRITE LOG-RECORD
TEST***       DISPLAY 'Article no longer on stock!'
TEST***       ACCEPT CONTINUE-KEY
           ELSE
              IF BACK-ART-AMOUNT > ART-AMOUNT
                 MOVE BACK-ORDER-NO TO LOG-ORDER-NO
                 MOVE BACK-CUST-NO TO LOG-CUST-NO
                 MOVE BACK-CUST-NAME TO LOG-CUST-NAME
                 MOVE BACK-ART-NO TO LOG-ITEM-NO
                 MOVE ERROR-MESSAGE-6 TO LOG-ITEM-STATUS
                 MOVE LOG-DATA TO LOG-RECORD
*****************PERFORM SEND-BACK-ORDER
                 WRITE LOG-RECORD
              ELSE
                 ADD 1 TO ORDER-ITEM-SENT
                 COMPUTE ART-AMOUNT = ART-AMOUNT
                         - BACK-ART-AMOUNT
                 IF ART-AMOUNT < ART-MINIMUM-AMOUNT
                    MOVE 'NO' TO ART-STATUS
                    PERFORM SEND-RESUPPLY-ITEM
                 END-IF
                 MOVE 'UPDATE '  TO DB-OPERATION
                 MOVE ARTICLE-RECORD TO DB-RECORD
                 MOVE BACK-ART-NO TO DB-SEARCH-KEY
                 CALL 'ARTICLE' USING DB-INTERFACE, DB-DIRECTORY
TEST***          DISPLAY 'Article ' DB-SEARCH-KEY ' was updated'
TEST***          ACCEPT CONTINUE-KEY
                 MOVE ZERO TO SEND-RETCODE
                 PERFORM SEND-DISPATCH-ORDER
                 PERFORM SEND-BILLING-ITEM
                 IF SEND-RETCODE = ZERO
                    MOVE BACK-ORDER-NO TO LOG-ORDER-NO
                    MOVE BACK-CUST-NO TO LOG-CUST-NO
                    MOVE BACK-CUST-NAME TO LOG-CUST-NAME
                    MOVE BACK-ART-NO TO LOG-ITEM-NO
                    MOVE ERROR-MESSAGE-11 TO LOG-ITEM-STATUS
                    MOVE LOG-DATA TO LOG-RECORD
                    WRITE LOG-RECORD
                    MOVE BACK-ORDER-NO TO AUFTRAGNR-OUT
                    MOVE BACK-CUST-NO TO KUNDENNR-OUT
                    MOVE BACK-CUST-NAME TO KUNDENNAME-OUT
                    MOVE BACK-ART-NO TO ARTNR-OUT
                    MOVE BACK-ART-AMOUNT TO MENGE-OUT
                    MOVE BACK-ART-NAME TO ARTNAME-OUT
                    MOVE PROTOKOLLDATEN TO PROTOKOLL-REC
                    WRITE PROTOKOLL-REC
                 END-IF
              END-IF
              ADD 1 TO ORDER-ITEM-COUNT
           END-IF
           EXIT.
      ****************************************************************
      * close out customer order
       TERMINATE-ORDER-PROCESSING.
      * close all XML files
***********CALL 'BACKORDR' USING SEND-FUNC, SEND-RETCODE,
***********                      BACK-ORDER, OUTPUT-DIRECTORY
***********CALL 'RESUPPLY' USING SEND-FUNC, SEND-RETCODE,
***********                      RESUPPLY-ITEM, OUTPUT-DIRECTORY
***********CALL 'DISPATCH' USING SEND-FUNC, SEND-RETCODE,
***********                      DISPATCH-ORDER, OUTPUT-DIRECTORY
***********CALL 'BILLING ' USING SEND-FUNC, SEND-RETCODE,
***********                      BILLING-ITEM, OUTPUT-DIRECTORY
           MOVE 'CLOS' TO SEND-FUNC
           MOVE BACK-ORDER-NO TO LOG-ORDER-NO
           MOVE BACK-CUST-NO TO LOG-CUST-NO
           MOVE BACK-CUST-NAME TO LOG-CUST-NAME
           MOVE ORDER-ITEM-COUNT TO LOG-ITEM-NO
           MOVE 'Order items were processed!' TO LOG-ITEM-STATUS
           MOVE LOG-DATA TO LOG-RECORD
           WRITE LOG-RECORD.
           MOVE ORDER-ITEM-SENT  TO LOG-ITEM-NO
           MOVE 'Order items fulfilled!' TO LOG-ITEM-STATUS
           MOVE LOG-DATA TO LOG-RECORD
           WRITE LOG-RECORD.
           CLOSE LOG-FILE
           EXIT.
      ****************************************************************
      * process the order item
       MESSAGE-SENDING SECTION.
      ****************************************************************
      * send out a back order if article amount not sufficient
      ****************************************************************
      * send out a resupply item if article amount below minimum amount
       SEND-RESUPPLY-ITEM.
TEST***    DISPLAY 'Send out Resupply Item!'
TEST***    ACCEPT CONTINUE-KEY.
           MOVE CURRENT-DATE TO RESUPPLY-DATE
           MOVE BACK-ART-NO TO RESUPPLY-ART-NO
           MOVE ART-TYPE TO RESUPPLY-ART-TYPE
           MOVE ART-RESUPPLY-AMOUNT TO RESUPPLY-ART-AMOUNT
           MOVE 'WRIT' TO SEND-FUNC
           MOVE ZERO TO SEND-RETCODE
           CALL 'SUPPLY'   USING SEND-FUNC, SEND-RETCODE,
                                 RESUPPLY-ITEM, OUTPUT-DIRECTORY
           IF SEND-RETCODE NOT = ZERO
              MOVE BACK-ORDER-NO TO LOG-ORDER-NO
              MOVE BACK-CUST-NO TO LOG-CUST-NO
              MOVE BACK-CUST-NAME TO LOG-CUST-NAME
              MOVE BACK-ART-NO TO LOG-ITEM-NO
              MOVE ERROR-MESSAGE-8 TO LOG-ITEM-STATUS
              MOVE LOG-DATA TO LOG-RECORD
              WRITE LOG-RECORD
TEST***       DISPLAY 'Supply Item was not sent RetCode = ' SEND-RETCODE
TEST***       ACCEPT CONTINUE-KEY
           END-IF.
           EXIT.
      ****************************************************************
      * send out a dispatch order if article amount is sufficient
       SEND-DISPATCH-ORDER.
TEST***    DISPLAY 'Send out Dispatch Order!'
TEST***    ACCEPT CONTINUE-KEY.
           MOVE CURRENT-DATE TO DISP-DATE
           MOVE BACK-ORDER-NO TO DISP-ORDER-NO
           MOVE BACK-CUST-NO TO DISP-CUST-NO
           MOVE BACK-CUST-NAME TO DISP-CUST-NAME
           MOVE BACK-DELIVERY-MODE TO DISP-DELIVERY-MODE
           MOVE CUST-STREET TO DISP-CUST-STREET
           MOVE CUST-ZIP    TO DISP-CUST-ZIP
           MOVE CUST-CITY   TO DISP-CUST-CITY
           MOVE CUST-STATE  TO DISP-CUST-STATE
           MOVE BACK-ITEM-NO TO DISP-ITEM-NO
           MOVE BACK-ART-NO  TO DISP-ART-NO
           MOVE BACK-ART-NAME TO DISP-ART-NAME
           MOVE BACK-ART-AMOUNT TO DISP-ART-AMOUNT
           MOVE 'WRIT' TO SEND-FUNC
           MOVE ZERO TO SEND-RETCODE
           CALL 'DISPATC ' USING SEND-FUNC, SEND-RETCODE,
                                 DISPATCH-ORDER, OUTPUT-DIRECTORY
           IF SEND-RETCODE NOT = ZERO
              MOVE BACK-ORDER-NO TO LOG-ORDER-NO
              MOVE BACK-CUST-NO TO LOG-CUST-NO
              MOVE BACK-CUST-NAME TO LOG-CUST-NAME
              MOVE BACK-ART-NO TO LOG-ITEM-NO
              MOVE ERROR-MESSAGE-9 TO LOG-ITEM-STATUS
              MOVE LOG-DATA TO LOG-RECORD
              WRITE LOG-RECORD
TEST***       DISPLAY 'Dispatch was not sent RetCode = ' SEND-RETCODE
TEST***       ACCEPT CONTINUE-KEY
           END-IF.
           EXIT.
      ****************************************************************
      * send out a billing item if article amount is sufficient
       SEND-BILLING-ITEM.
TEST***    DISPLAY 'Send out Billing Item!'
TEST***    ACCEPT CONTINUE-KEY.
           MOVE CURRENT-DATE TO BILL-DATE
           MOVE BACK-ORDER-NO TO BILL-ORDER-NO
           MOVE BACK-CUST-NO TO BILL-CUST-NO
           MOVE BACK-ITEM-NO TO BILL-ITEM-NO
           MOVE BACK-ART-NO  TO BILL-ART-NO
           MOVE BACK-ART-NAME TO BILL-ART-NAME
           MOVE BACK-ART-AMOUNT TO BILL-ART-AMOUNT
           MOVE '????' TO BILL-PAYMENT-MODE
           MOVE ART-PRICE TO BILL-ART-PRICE
           MOVE 'WRIT' TO SEND-FUNC
           MOVE ZERO TO SEND-RETCODE
           CALL 'BILLING ' USING SEND-FUNC, SEND-RETCODE,
                                 BILLING-ITEM, OUTPUT-DIRECTORY
           IF SEND-RETCODE NOT = ZERO
              MOVE BACK-ORDER-NO TO LOG-ORDER-NO
              MOVE BACK-CUST-NO TO LOG-CUST-NO
              MOVE BACK-CUST-NAME TO LOG-CUST-NAME
              MOVE BACK-ART-NO TO LOG-ITEM-NO
              MOVE ERROR-MESSAGE-10 TO LOG-ITEM-STATUS
              MOVE LOG-DATA TO LOG-RECORD
              WRITE LOG-RECORD
TEST***       DISPLAY 'Bill Item was not sent RetCode = ' SEND-RETCODE
TEST***       ACCEPT CONTINUE-KEY
           END-IF.
           EXIT.
      ******************************************************************
      ********************** end of source *****************************
